module ORMakers
  # Database model module
  module Model
    def self.create_table_for_model(model)
      Database.query("CREATE TABLE #{model.name.snake_case}s (id serial);")
    rescue PG::DuplicateTable
      'Model table already exists'
    end

    def self.included(model_class)
      create_table_for_model(model_class)
      model_class.send(:include, InstanceMethods)
      model_class.extend(ClassMethods)
    end

    # Instance methods
    module InstanceMethods
      def initialize(attributes)
        create_instance_vars(attributes)
        create_attr_readers
      end

      def destroy
        Database.query("DELETE FROM #{self.class.name.snake_case}s WHERE id='#{id}';")
      end

      def create_instance_vars(attributes)
        attributes.each { |name, val| instance_variable_set("@#{name}", val) }
      end

      def create_attr_readers
        instance_variables.each do |var|
          self.class.send(:attr_reader, var.to_s[1..-1].to_sym)
        end
      end
    end

    # Class methods
    module ClassMethods
      def create(attributes = {})
        attr_keys = attributes.keys.map(&:to_s).join(', ')
        attr_values = attributes.values.map { |val| "'#{val}'" }.join(', ')
        id = Database.query("INSERT INTO #{name.snake_case}s (#{attr_keys}) VALUES (#{attr_values}) returning *;").first['id']
        attributes['id'] = id
        new(attributes)
      end

      def property(name, type)
        Database.query("ALTER TABLE #{self.name.snake_case}s ADD #{name} #{Database::DATA_TYPES[type]};")
      rescue PG::DuplicateColumn
        'Property column already exists'
      end

      def all
        results = Database.query("SELECT * FROM #{self.name.snake_case}s;")
        results.map { |result| new(result) }
      end

      def get(id)
        result = Database.query("SELECT * FROM #{self.name.snake_case}s WHERE id='#{id}';").first
        new(result)
      end
    end
  end
end